@@ -26,8 +26,10 @@
 #include <string.h>
 #include <stdint.h>
 #include <stdio.h>
+#include <pwd.h>
 
 #include "qmi-utils.h"
+#include "qmi-error-types.h"
 
 /**
  * SECTION:qmi-utils
@@ -75,6 +77,46 @@ __qmi_utils_str_hex (gconstpointer mem,
 
 /*****************************************************************************/
 
+gboolean
+__qmi_user_allowed (uid_t uid,
+                    GError **error)
+{
+#ifndef QMI_USERNAME_ENABLED
+    if (uid == 0)
+        return TRUE;
+#else
+# ifndef QMI_USERNAME
+#  error QMI username not defined
+# endif
+
+    struct passwd *expected_usr = NULL;
+
+    /* Root user is always allowed, regardless of the specified QMI_USERNAME */
+    if (uid == 0)
+        return TRUE;
+
+    expected_usr = getpwnam (QMI_USERNAME);
+    if (!expected_usr) {
+        g_set_error (error,
+                     QMI_CORE_ERROR,
+                     QMI_CORE_ERROR_FAILED,
+                     "Not enough privileges (unknown username %s)", QMI_USERNAME);
+        return FALSE;
+    }
+
+    if (uid == expected_usr->pw_uid)
+        return TRUE;
+#endif
+
+    g_set_error (error,
+                 QMI_CORE_ERROR,
+                 QMI_CORE_ERROR_FAILED,
+                 "Not enough privileges");
+    return FALSE;
+}
+
+/*****************************************************************************/
+
 #if defined UTILS_ENABLE_TRACE
 static void
 print_read_bytes_trace (const gchar *type,
@@ -909,9 +951,9 @@ qmi_utils_read_string_from_buffer (const guint8 **buffer,
  * @buffer: a buffer with raw binary data.
  * @buffer_size: size of @buffer.
  * @fixed_size: number of bytes to read.
- * @out: return location for the read string. The returned value should be freed with g_free().
+ * @out: buffer preallocated by the client, with at least @fixed_size bytes.
  *
- * Reads a @fixed_size-sized string from the buffer.
+ * Reads a @fixed_size-sized string from the buffer into the @out buffer.
  *
  * Also note that both @buffer and @buffer_size get updated after the
  * @fixed_size bytes read.
@@ -956,7 +998,7 @@ qmi_utils_write_string_to_buffer (guint8      **buffer,
                                   guint8        length_prefix_size,
                                   const gchar  *in)
 {
-    guint16 len;
+    gsize len;
     guint8 len_8;
     guint16 len_16;
 
@@ -967,20 +1009,26 @@ qmi_utils_write_string_to_buffer (guint8      **buffer,
               length_prefix_size == 8 ||
               length_prefix_size == 16);
 
-    len = (guint16) strlen (in);
+    len = strlen (in);
+
+    g_assert (   len + (length_prefix_size/8) <= *buffer_size
+              || (length_prefix_size==8 && ((int) G_MAXUINT8 + 1) < *buffer_size));
 
     switch (length_prefix_size) {
     case 0:
         break;
     case 8:
-        g_warn_if_fail (len <= G_MAXUINT8);
+        if (len > G_MAXUINT8) {
+            g_warn_if_reached ();
+            len = G_MAXUINT8;
+        }
         len_8 = (guint8)len;
         qmi_utils_write_guint8_to_buffer (buffer,
                                           buffer_size,
                                           &len_8);
         break;
     case 16:
-        g_warn_if_fail (len <= G_MAXUINT16);
+        /* already asserted that @len is no larger then @buffer_size */
         len_16 = (guint16)len;
         qmi_utils_write_guint16_to_buffer (buffer,
                                            buffer_size,
@@ -1021,6 +1069,7 @@ qmi_utils_write_fixed_size_string_to_buffer (guint8      **buffer,
     g_assert (buffer != NULL);
     g_assert (buffer_size != NULL);
     g_assert (fixed_size > 0);
+    g_assert (fixed_size <= *buffer_size);
 
     memcpy (*buffer, in, fixed_size);
     *buffer = &((*buffer)[fixed_size]);
