@@ -29,22 +29,25 @@ gchar *qmicli_get_raw_data_printable (const GArray *data,
                                       gsize max_line_length,
                                       const gchar *new_line_prefix);
 
-gboolean qmicli_read_pin_id_from_string         (const gchar *str,
-                                                 QmiDmsUimPinId *out);
-gboolean qmicli_read_operating_mode_from_string (const gchar *str,
-                                                 QmiDmsOperatingMode *out);
-gboolean qmicli_read_rat_mode_pref_from_string  (const gchar *str,
-                                                 QmiNasRatModePreference *out);
-gboolean qmicli_read_facility_from_string       (const gchar *str,
-                                                 QmiDmsUimFacility *out);
-gboolean qmicli_read_enable_disable_from_string (const gchar *str,
-                                                 gboolean *out);
-gboolean qmicli_read_firmware_id_from_string    (const gchar *str,
-                                                 QmiDmsFirmwareImageType *out_type,
-                                                 guint *out_index);
-
-gboolean qmicli_read_radio_interface_from_string (const gchar *str,
-                                                  QmiNasRadioInterface *out);
+gboolean qmicli_read_pin_id_from_string              (const gchar *str,
+                                                      QmiDmsUimPinId *out);
+gboolean qmicli_read_operating_mode_from_string      (const gchar *str,
+                                                      QmiDmsOperatingMode *out);
+gboolean qmicli_read_rat_mode_pref_from_string       (const gchar *str,
+                                                      QmiNasRatModePreference *out);
+gboolean qmicli_read_facility_from_string            (const gchar *str,
+                                                      QmiDmsUimFacility *out);
+gboolean qmicli_read_enable_disable_from_string      (const gchar *str,
+                                                      gboolean *out);
+gboolean qmicli_read_firmware_id_from_string         (const gchar *str,
+                                                      QmiDmsFirmwareImageType *out_type,
+                                                      guint *out_index);
+gboolean qmicli_read_radio_interface_from_string     (const gchar *str,
+                                                      QmiNasRadioInterface *out);
+gboolean qmicli_read_net_open_flags_from_string      (const gchar *str,
+                                                      QmiDeviceOpenFlags *out);
+gboolean qmicli_read_link_layer_protocol_from_string (const gchar *str,
+                                                      QmiWdaLinkLayerProtocol *out);
 
 gboolean qmicli_read_non_empty_string           (const gchar *str,
                                                  const gchar *description,
